// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "RequestQueue.h"

#include "Request.h"

#include <hu/varadiistvan/xplcommon/Util.h>

//------------------------------------------------------------------------------

using xplra::RequestQueue;

using hu::varadiistvan::xplcommon::Util;

//------------------------------------------------------------------------------

float RequestQueue::flightLoop(float /*inElapsedSinceLastCall*/,
                               float /*inElapsedTimeSinceLastFlightLoop*/,
                               int /*inCounter*/, void* inRefCon)
{
    RequestQueue* requestQueue = reinterpret_cast<RequestQueue*>(inRefCon);
    requestQueue->run();
#if USE_SCHEDULE_FLIGHTLOOP
    return 0.0;
#else
    return flightLoopInterval;
#endif
}

//------------------------------------------------------------------------------

RequestQueue::RequestQueue() :
    enabled(true)
{
#if USE_SCHEDULE_FLIGHTLOOP
    XPLMCreateFlightLoop_t createFL;

    createFL.structSize = sizeof(createFL);
    createFL.phase = xplm_FlightLoop_Phase_AfterFlightModel;
    createFL.callbackFunc = &flightLoop;
    createFL.refcon = this;

    flightLoopID = XPLMCreateFlightLoop(&createFL);
#else
    XPLMRegisterFlightLoopCallback(&flightLoop, flightLoopInterval, this);
#endif
}

//------------------------------------------------------------------------------

bool RequestQueue::execute(Request* request)
{
    mutex.lock();
    if (!enabled) {
        mutex.unlock();
        return false;
    }

    request->completed = false;
    requests.push_back(request);

#if USE_SCHEDULE_FLIGHTLOOP
    if (requests.size()==1) {
        XPLMScheduleFlightLoop(flightLoopID, -2.0, true);
    }
#endif

    while(enabled && !request->completed) {
        requestsDone.wait(mutex);
    }

    bool result = request->completed;

    mutex.unlock();

    return result;
}

//------------------------------------------------------------------------------

void RequestQueue::disable()
{
    mutex.lock();
    enabled = false;
    mutex.unlock();
    requestsDone.broadcast();
#if USE_SCHEDULE_FLIGHTLOOP
    XPLMDestroyFlightLoop(flightLoopID);
#else
    XPLMUnregisterFlightLoopCallback(&flightLoop, this);
#endif
}

//------------------------------------------------------------------------------

void RequestQueue::run()
{
    requests_t queuedRequests;

    mutex.lock();
    requests.swap(queuedRequests);
    mutex.unlock();

    if (queuedRequests.empty()) return;

    for(requests_t::const_iterator i = queuedRequests.begin();
        i!=queuedRequests.end(); ++i)
    {
        (*i)->execute();
    }

    mutex.lock();
    for(requests_t::const_iterator i = queuedRequests.begin();
        i!=queuedRequests.end(); ++i)
    {
        (*i)->completed = true;
    }
    mutex.unlock();
    requestsDone.broadcast();
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
